// Conversion and enhancement by Access Software Technologies

{* Copyright (c) 1992-1995, Microsoft Corporation, all rights reserved
**
** ras.h
** Remote Access external API
** Public header for external API clients
*}

unit Ras3;

interface

uses Windows, Messages, SysUtils, Dialogs;

const

  DNLEN = 15;  // Maximum domain name length
  UNLEN = 256; // Maximum user name length
  PWLEN = 256; // Maximum password length
  NETBIOS_NAME_LEN = 16; // NetBIOS net name (bytes)

  RAS_MaxDeviceType  = 16;
  RAS_MaxPhoneNumber = 128;
  RAS_MaxIpAddress   = 15;
  RAS_MaxIpxAddress  = 21;


{$IFDEF WINVER31}
{Version 3.x sizes }
  RAS_MaxEntryName      = 20;
  RAS_MaxDeviceName     = 32;
  RAS_MaxCallbackNumber = 48;
{$ELSE}
{Version 4.x sizes }
  RAS_MaxEntryName      = 256;
  RAS_MaxDeviceName     = 128;
  RAS_MaxCallbackNumber = RAS_MaxPhoneNumber;
{$ENDIF}

type

  LPHRasConn = ^THRasConn;
  THRasConn = Longint;


{* Identifies an active RAS connection.  (See RasEnumConnections)
*}
  LPRasConnW = ^TRasConnW;
  TRasConnW = record
    dwSize: Longint;
    hrasconn: THRasConn;
    szEntryName: Array[0..RAS_MaxEntryName] of WideChar;
{$IFNDEF WINVER31}
   szDeviceType: Array[0..RAS_MaxDeviceType] of WideChar;
   szDeviceName: Array[0..RAS_MaxDeviceName] of WideChar;
{$ENDIF}
  end;

  LPRasConnA = ^TRasConnA;
  TRasConnA = record
    dwSize: Longint;
    hrasconn: THRasConn;
    szEntryName: Array[0..RAS_MaxEntryName] of AnsiChar;
{$IFNDEF WINVER31}
   szDeviceType: Array[0..RAS_MaxDeviceType] of AnsiChar;
   szDeviceName: Array[0..RAS_MaxDeviceName] of AnsiChar;
{$ENDIF}
  end;

  LPRasConn = ^TRasConn;
  TRasConn = TRasConnA;


const

{* Enumerates intermediate states to a connection.  (See RasDial)
*}
  RASCS_PAUSED = $1000;
  RASCS_DONE   = $2000;

type

  LPRasConnState = ^TRasConnState;
  TRasConnState = Integer;

const

  RASCS_OpenPort = 0;
  RASCS_PortOpened = 1;
  RASCS_ConnectDevice = 2;
  RASCS_DeviceConnected = 3;
  RASCS_AllDevicesConnected = 4;
  RASCS_Authenticate = 5;
  RASCS_AuthNotify = 6;
  RASCS_AuthRetry = 7;
  RASCS_AuthCallback = 8;
  RASCS_AuthChangePassword = 9;
  RASCS_AuthProject = 10;
  RASCS_AuthLinkSpeed = 11;
  RASCS_AuthAck = 12;
  RASCS_ReAuthenticate = 13;
  RASCS_Authenticated = 14;
  RASCS_PrepareForCallback = 15;
  RASCS_WaitForModemReset = 16;
  RASCS_WaitForCallback = 17;
  RASCS_Projected = 18;
{$IFNDEF WINVER31}
  RASCS_StartAuthentication = 19;
  RASCS_CallbackComplete = 20;
  RASCS_LogonNetwork = 21;
{$ENDIF}

  RASCS_Interactive = RASCS_PAUSED;
  RASCS_RetryAuthentication = RASCS_PAUSED + 1;
  RASCS_CallbackSetByCaller = RASCS_PAUSED + 2;
  RASCS_PasswordExpired = RASCS_PAUSED + 3;

  RASCS_Connected = RASCS_DONE;
  RASCS_Disconnected = RASCS_DONE + 1;

type

{* Describes the status of a RAS connection.  (See RasConnectionStatus)
*}
  LPRasConnStatusW = ^TRasConnStatusW;
  TRasConnStatusW = record
    dwSize: Longint;
    rasconnstate: TRasConnState;
    dwError: LongInt;
    szDeviceType: Array[0..RAS_MaxDeviceType] of WideChar;
    szDeviceName: Array[0..RAS_MaxDeviceName] of WideChar;
  end;

  LPRasConnStatusA = ^TRasConnStatusA;
  TRasConnStatusA = record
    dwSize: Longint;
    rasconnstate: TRasConnState;
    dwError: LongInt;
    szDeviceType: Array[0..RAS_MaxDeviceType] of AnsiChar;
    szDeviceName: Array[0..RAS_MaxDeviceName] of AnsiChar;
  end;

  LPRasConnStatus = ^TRasConnStatus;
  TRasConnStatus = TRasConnStatusA;


{* Describes connection establishment parameters.  (See RasDial)
*}
  LPRasDialParamsW = ^TRasDialParamsW;
  TRasDialParamsW = record
    dwSize: LongInt;
    szEntryName: Array[0..RAS_MaxEntryName] of WideChar;
    szPhoneNumber: Array[0..RAS_MaxPhoneNumber] of WideChar;
    szCallbackNumber: Array[0..RAS_MaxCallbackNumber] of WideChar;
    szUserName: Array[0..UNLEN] of WideChar;
    szPassword: Array[0..PWLEN] of WideChar;
    szDomain: Array[0..DNLEN] of WideChar;
  end;

  LPRasDialParamsA = ^TRasDialParamsA;
  TRasDialParamsA = record
    dwSize: LongInt;
    szEntryName: Array[0..RAS_MaxEntryName] of AnsiChar;
    szPhoneNumber: Array[0..RAS_MaxPhoneNumber] of AnsiChar;
    szCallbackNumber: Array[0..RAS_MaxCallbackNumber] of AnsiChar;
    szUserName: Array[0..UNLEN] of AnsiChar;
    szPassword: Array[0..PWLEN] of AnsiChar;
    szDomain: Array[0..DNLEN] of AnsiChar;
  end;

  LPRasDialParams = ^TRasDialParams;
  TRasDialParams = TRasDialParamsA;


{* Describes extended connection establishment options.  (See RasDial)
*}
  LPRasDialExtensions = ^TRasDialExtensions;
  TRasDialExtensions = record
    dwSize: LongInt;
    dwfOptions: LongInt;
    hwndParent: HWND;
    reserved: LongInt;
  end;

const

{* 'dwfOptions' bit flags.
*}
  RDEOPT_UsePrefixSuffix           = $00000001;
  RDEOPT_PausedStates              = $00000002;
  RDEOPT_IgnoreModemSpeaker        = $00000004;
  RDEOPT_SetModemSpeaker           = $00000008;
  RDEOPT_IgnoreSoftwareCompression = $00000010;
  RDEOPT_SetSoftwareCompression    = $00000020;


type

{* Describes an enumerated RAS phone book entry name.  (See RasEntryEnum)
*}
  LPRasEntryNameW = ^TRasEntryNameW;
  TRasEntryNameW = record
    dwSize: Longint;
    szEntryName: Array[0..RAS_MaxEntryName] of WideChar;
  end;

  LPRasEntryNameA = ^TRasEntryNameA;
  TRasEntryNameA = record
    dwSize: Longint;
    szEntryName: Array[0..RAS_MaxEntryName] of AnsiChar;
  end;

  LPRasEntryName = ^TRasEntryName;
  TRasEntryName = TRasEntryNameA;


{* Protocol code to projection data structure mapping.
*}
  LPRasProjection = ^TRasProjection;
  TRasProjection = Integer;

const

  RASP_Amb = $10000;
  RASP_PppNbf = $803F;
  RASP_PppIpx = $802B;
  RASP_PppIp = $8021;


type

{* Describes the result of a RAS AMB (Authentication Message Block)
** projection.  This protocol is used with NT 3.1 and OS/2 1.3 downlevel
** RAS servers.
*}
  LPRasAmbW = ^TRasAmbW;
  TRasAmbW = record
    dwSize: Longint;
    dwError: Longint;
    szNetBiosError: Array[0..NETBIOS_NAME_LEN] of WideChar;
    bLana: Byte;
  end;

  LPRasAmbA = ^TRasAmbA;
  TRasAmbA = record
    dwSize: Longint;
    dwError: Longint;
    szNetBiosError: Array[0..NETBIOS_NAME_LEN] of AnsiChar;
    bLana: Byte;
  end;

  LPRasAmb = ^TRasAmb;
  TRasAmb = TRasAmbA;


{* Describes the result of a PPP NBF (NetBEUI) projection.
*}
  LPRasPppNbfW = ^TRasPppNbfW;
  TRasPppNbfW = record
    dwSize: Longint;
    dwError: Longint;
    dwNetBiosError: Longint;
    szNetBiosError: Array[0..NETBIOS_NAME_LEN] of WideChar;
    szWorkstationName: Array[0..NETBIOS_NAME_LEN] of WideChar;
    bLana: Byte;
  end;

  LPRasPppNbfA = ^TRasPppNbfA;
  TRasPppNbfA = record
    dwSize: Longint;
    dwError: Longint;
    dwNetBiosError: Longint;
    szNetBiosError: Array[0..NETBIOS_NAME_LEN] of AnsiChar;
    szWorkstationName: Array[0..NETBIOS_NAME_LEN] of AnsiChar;
    bLana: Byte;
  end;

  LpRaspppNbf = ^TRasPppNbf;
  TRasPppNbf = TRasPppNbfA;


{* Describes the results of a PPP IPX (Internetwork Packet Exchange)
** projection.
*}
  LPRasPppIpxW = ^TRasPppIpxW;
  TRasPppIpxW = record
    dwSize: Longint;
    dwError: Longint;
    szIpxAddress: Array[0..RAS_MaxIpxAddress] of WideChar;
  end;

  LPRasPppIpxA = ^TRasPppIpxA;
  TRasPppIpxA = record
    dwSize: Longint;
    dwError: Longint;
    szIpxAddress: Array[0..RAS_MaxIpxAddress] of AnsiChar;
  end;

  LPRasPppIpx = ^TRasPppIpx;
  TRasPppIpx = TRasPppIpxA;


{* Describes the results of a PPP IP (Internet) projection.
*}
  LPRasPppIpW = ^TRasPppIpW;
  TRasPppIpW = record
    dwSize: Longint;
    dwError: Longint;
    szIpAddress: Array[0..RAS_MaxIpAddress] of WideChar;

{$IFNDEF WINNT35COMPATIBLE}

    {* This field was added between Windows NT 3.51 beta and Windows NT 3.51
    ** final, and between Windows 95 M8 beta and Windows 95 final.  If you do
    ** not require the server address and wish to retrieve PPP IP information
    ** from Windows NT 3.5 or early Windows NT 3.51 betas, or on early Windows
    ** 95 betas, define WINNT35COMPATIBLE.
    **
    ** The server IP address is not provided by all PPP implementations,
    ** though Windows NT server's do provide it.
    *}
    szServerIpAddress: Array[0..RAS_MaxIpAddress] of WideChar;

{$ENDIF}
  end;

  LPRasPppIpA = ^TRasPppIpA;
  TRasPppIpA = record
    dwSize: Longint;
    dwError: Longint;
    szIpAddress: Array[0..RAS_MaxIpAddress] of AnsiChar;

{$IFNDEF WINNT35COMPATIBLE}

    {* See RASPPPIPW comment.
    *}
    szServerIpAddress: Array[0..RAS_MaxIpAddress] of AnsiChar;

{$ENDIF}
  end;

  LPRasPppIp = ^TRasPppIp;
  TRasPppIp = TRasPppIpA;


const

{* If using RasDial message notifications, get the notification message code
** by passing this string to the RegisterWindowMessageA() API.
** WM_RASDIALEVENT is used only if a unique message cannot be registered.
*}
  RASDIALEVENT    = 'RasDialEvent';
  WM_RASDIALEVENT = $CCCD;


{* Prototypes for caller's RasDial callback handler.  Arguments are the
** message ID (currently always WM_RASDIALEVENT), the current RASCONNSTATE and
** the error that has occurred (or 0 if none).  Extended arguments are the
** handle of the RAS connection and an extended error code.
*}
{
typedef VOID (WINAPI *RASDIALFUNC)( UINT, RASCONNSTATE, DWORD );
typedef VOID (WINAPI *RASDIALFUNC1)( HRASCONN, UINT, RASCONNSTATE, DWORD, DWORD );
}

{* External RAS API function prototypes.
*}

type
  TRasDialA = function (
    lpRasDialExt: LPRasDialExtensions;
    lpszPhoneBook: PAnsiChar;
    var params: TRasDialParamsA;
    dwNotifierType: Longint;
    lpNotifier: DWord;
    var rasconn: THRasConn
    ): Longint; stdcall;
  TRasDialW = function (
    lpRasDialExt: LPRasDialExtensions;
    lpszPhoneBook: PWideChar;
    var params: TRasDialParamsW;
    dwNotifierType: Longint;
    lpNotifier: DWord;
    var rasconn: THRasConn
    ): Longint; stdcall;
  TRasDial = function (
    lpRasDialExt: LPRasDialExtensions;
    lpszPhoneBook: PAnsiChar;
    var params: TRasDialParams;
    dwNotifierType: Longint;
    lpNotifier: DWord;
    var rasconn: THRasConn
    ): Longint; stdcall;

  TRasEnumConnectionsA = function (
    rasconnArray: LPRasConnA;
    var lpcb: Longint;
    var lpcConnections: Longint
    ): Longint; stdcall;
  TRasEnumConnectionsW = function (
    rasconnArray: LPRasConnW;
    var lpcb: Longint;
    var lpcConnections: Longint
    ): Longint; stdcall;
  TRasEnumConnections = function (
    rasconnArray: LPRasConn;
    var lpcb: Longint;
    var lpcConnections: Longint
    ): Longint; stdcall;

  TRasEnumEntriesA = function (
    reserved: PAnsiChar;
    lpszPhoneBook: PAnsiChar;
    entrynamesArray: LPRasEntryNameA;
    var lpcb: Longint;
    var lpcEntries: Longint
    ): Longint; stdcall;
  TRasEnumEntriesW = function (
    reserved: PWideChar;
    lpszPhoneBook: PWideChar;
    entrynamesArray: LPRasEntryNameW;
    var lpcb: Longint;
    var lpcEntries: Longint
    ): Longint; stdcall;
  TRasEnumEntries = function (
    reserved: PAnsiChar;
    lpszPhoneBook: PAnsiChar;
    entrynamesArray: LPRasEntryName;
    var lpcb: Longint;
    var lpcEntries: Longint
    ): Longint; stdcall;

  TRasGetConnectStatusA = function (
    hConn: THRasConn;
    var lpStatus: TRasConnStatusA
    ): Longint; stdcall;
  TRasGetConnectStatusW = function (
    hConn: THRasConn;
    var lpStatus: TRasConnStatusW
    ): Longint; stdcall;
  TRasGetConnectStatus = function (
    hConn: THRasConn;
    var lpStatus: TRasConnStatus
    ): Longint; stdcall;

  TRasGetErrorStringA = function (
    errorValue: Integer;
    erroString: PAnsiChar;
    cBufSize: Longint
    ): Longint; stdcall;
  TRasGetErrorStringW = function (
    errorValue: Integer;
    erroString: PWideChar;
    cBufSize: Longint
    ): Longint; stdcall;
  TRasGetErrorString = function (
    errorValue: Integer;
    erroString: PAnsiChar;
    cBufSize: Longint
    ): Longint; stdcall;

  TRasHangUpA = function (
    hConn: THRasConn
    ): Longint; stdcall;
  TRasHangUpW = function (
    hConn: THRasConn
    ): Longint; stdcall;
  TRasHangUp = function (
    hConn: THRasConn
    ): Longint; stdcall;

  TRasGetProjectionInfoA = function (
    hConn: THRasConn;
    rasproj: TRasProjection;
    lpProjection: Pointer;
    var lpcb: Longint
    ): Longint; stdcall;
  TRasGetProjectionInfoW = function (
    hConn: THRasConn;
    rasproj: TRasProjection;
    lpProjection: Pointer;
    var lpcb: Longint
    ): Longint; stdcall;
  TRasGetProjectionInfo = function (
    hConn: THRasConn;
    rasproj: TRasProjection;
    lpProjection: Pointer;
    var lpcb: Longint
    ): Longint; stdcall;

  TRasCreatePhonebookEntryA = function (
    hwndParentWindow: HWND;
    lpszPhoneBook: PAnsiChar
    ): Longint; stdcall;
  TRasCreatePhonebookEntryW = function (
    hwndParentWindow: HWND;
    lpszPhoneBook: PWideChar
    ): Longint; stdcall;
  TRasCreatePhonebookEntry = function (
    hwndParentWindow: HWND;
    lpszPhoneBook: PAnsiChar
    ): Longint; stdcall;

  TRasEditPhonebookEntryA = function (
    hwndParentWindow: HWND;
    lpszPhoneBook: PAnsiChar;
    lpszEntryName: PAnsiChar
    ): Longint; stdcall;
  TRasEditPhonebookEntryW = function (
    hwndParentWindow: HWND;
    lpszPhoneBook: PWideChar;
    lpszEntryName: PWideChar
    ): Longint; stdcall;
  TRasEditPhonebookEntry = function (
    hwndParentWindow: HWND;
    lpszPhoneBook: PAnsiChar;
    lpszEntryName: PAnsiChar
    ): Longint; stdcall;

  TRasSetEntryDialParamsA = function (
    lpszPhoneBook: PAnsiChar;
    var lpDialParams: TRasDialParamsA;
    fRemovePassword: LongBool
    ): Longint; stdcall;
  TRasSetEntryDialParamsW = function (
    lpszPhoneBook: PWideChar;
    var lpDialParams: TRasDialParamsW;
    fRemovePassword: LongBool
    ): Longint; stdcall;
  TRasSetEntryDialParams = function (
    lpszPhoneBook: PAnsiChar;
    var lpDialParams: TRasDialParams;
    fRemovePassword: LongBool
    ): Longint; stdcall;

  TRasGetEntryDialParamsA = function (
    lpszPhoneBook: PAnsiChar;
    var lpDialParams: TRasDialParamsA;
    var lpfPassword: LongBool
    ): Longint; stdcall;
  TRasGetEntryDialParamsW = function (
    lpszPhoneBook: PWideChar;
    var lpDialParams: TRasDialParamsW;
    var lpfPassword: LongBool
    ): Longint; stdcall;
  TRasGetEntryDialParams = function (
    lpszPhoneBook: PAnsiChar;
    var lpDialParams: TRasDialParams;
    var lpfPassword: LongBool
    ): Longint; stdcall;


{**
** raserror.h
** Remote Access external API
** RAS specific error codes
*}

const

  RASBASE = 600;
  SUCCESS = 0;

  PENDING                              = (RASBASE+0);
  ERROR_INVALID_PORT_HANDLE            = (RASBASE+1);
  ERROR_PORT_ALREADY_OPEN              = (RASBASE+2);
  ERROR_BUFFER_TOO_SMALL               = (RASBASE+3);
  ERROR_WRONG_INFO_SPECIFIED           = (RASBASE+4);
  ERROR_CANNOT_SET_PORT_INFO           = (RASBASE+5);
  ERROR_PORT_NOT_CONNECTED             = (RASBASE+6);
  ERROR_EVENT_INVALID                  = (RASBASE+7);
  ERROR_DEVICE_DOES_NOT_EXIST          = (RASBASE+8);
  ERROR_DEVICETYPE_DOES_NOT_EXIST      = (RASBASE+9);
  ERROR_BUFFER_INVALID                 = (RASBASE+10);
  ERROR_ROUTE_NOT_AVAILABLE            = (RASBASE+11);
  ERROR_ROUTE_NOT_ALLOCATED            = (RASBASE+12);
  ERROR_INVALID_COMPRESSION_SPECIFIED  = (RASBASE+13);
  ERROR_OUT_OF_BUFFERS                 = (RASBASE+14);
  ERROR_PORT_NOT_FOUND                 = (RASBASE+15);
  ERROR_ASYNC_REQUEST_PENDING          = (RASBASE+16);
  ERROR_ALREADY_DISCONNECTING          = (RASBASE+17);
  ERROR_PORT_NOT_OPEN                  = (RASBASE+18);
  ERROR_PORT_DISCONNECTED              = (RASBASE+19);
  ERROR_NO_ENDPOINTS                   = (RASBASE+20);
  ERROR_CANNOT_OPEN_PHONEBOOK          = (RASBASE+21);
  ERROR_CANNOT_LOAD_PHONEBOOK          = (RASBASE+22);
  ERROR_CANNOT_FIND_PHONEBOOK_ENTRY    = (RASBASE+23);
  ERROR_CANNOT_WRITE_PHONEBOOK         = (RASBASE+24);
  ERROR_CORRUPT_PHONEBOOK              = (RASBASE+25);
  ERROR_CANNOT_LOAD_STRING             = (RASBASE+26);
  ERROR_KEY_NOT_FOUND                  = (RASBASE+27);
  ERROR_DISCONNECTION                  = (RASBASE+28);
  ERROR_REMOTE_DISCONNECTION           = (RASBASE+29);
  ERROR_HARDWARE_FAILURE               = (RASBASE+30);
  ERROR_USER_DISCONNECTION             = (RASBASE+31);
  ERROR_INVALID_SIZE                   = (RASBASE+32);
  ERROR_PORT_NOT_AVAILABLE             = (RASBASE+33);
  ERROR_CANNOT_PROJECT_CLIENT          = (RASBASE+34);
  ERROR_UNKNOWN                        = (RASBASE+35);
  ERROR_WRONG_DEVICE_ATTACHED          = (RASBASE+36);
  ERROR_BAD_STRING                     = (RASBASE+37);
  ERROR_REQUEST_TIMEOUT                = (RASBASE+38);
  ERROR_CANNOT_GET_LANA                = (RASBASE+39);
  ERROR_NETBIOS_ERROR                  = (RASBASE+40);
  ERROR_SERVER_OUT_OF_RESOURCES        = (RASBASE+41);
  ERROR_NAME_EXISTS_ON_NET             = (RASBASE+42);
  ERROR_SERVER_GENERAL_NET_FAILURE     = (RASBASE+43);
  WARNING_MSG_ALIAS_NOT_ADDED          = (RASBASE+44);
  ERROR_AUTH_INTERNAL                  = (RASBASE+45);
  ERROR_RESTRICTED_LOGON_HOURS         = (RASBASE+46);
  ERROR_ACCT_DISABLED                  = (RASBASE+47);
  ERROR_PASSWD_EXPIRED                 = (RASBASE+48);
  ERROR_NO_DIALIN_PERMISSION           = (RASBASE+49);
  ERROR_SERVER_NOT_RESPONDING          = (RASBASE+50);
  ERROR_FROM_DEVICE                    = (RASBASE+51);
  ERROR_UNRECOGNIZED_RESPONSE          = (RASBASE+52);
  ERROR_MACRO_NOT_FOUND                = (RASBASE+53);
  ERROR_MACRO_NOT_DEFINED              = (RASBASE+54);
  ERROR_MESSAGE_MACRO_NOT_FOUND        = (RASBASE+55);
  ERROR_DEFAULTOFF_MACRO_NOT_FOUND     = (RASBASE+56);
  ERROR_FILE_COULD_NOT_BE_OPENED       = (RASBASE+57);
  ERROR_DEVICENAME_TOO_LONG            = (RASBASE+58);
  ERROR_DEVICENAME_NOT_FOUND           = (RASBASE+59);
  ERROR_NO_RESPONSES                   = (RASBASE+60);
  ERROR_NO_COMMAND_FOUND               = (RASBASE+61);
  ERROR_WRONG_KEY_SPECIFIED            = (RASBASE+62);
  ERROR_UNKNOWN_DEVICE_TYPE            = (RASBASE+63);
  ERROR_ALLOCATING_MEMORY              = (RASBASE+64);
  ERROR_PORT_NOT_CONFIGURED            = (RASBASE+65);
  ERROR_DEVICE_NOT_READY               = (RASBASE+66);
  ERROR_READING_INI_FILE               = (RASBASE+67);
  ERROR_NO_CONNECTION                  = (RASBASE+68);
  ERROR_BAD_USAGE_IN_INI_FILE          = (RASBASE+69);
  ERROR_READING_SECTIONNAME            = (RASBASE+70);
  ERROR_READING_DEVICETYPE             = (RASBASE+71);
  ERROR_READING_DEVICENAME             = (RASBASE+72);
  ERROR_READING_USAGE                  = (RASBASE+73);
  ERROR_READING_MAXCONNECTBPS          = (RASBASE+74);
  ERROR_READING_MAXCARRIERBPS          = (RASBASE+75);
  ERROR_LINE_BUSY                      = (RASBASE+76);
  ERROR_VOICE_ANSWER                   = (RASBASE+77);
  ERROR_NO_ANSWER                      = (RASBASE+78);
  ERROR_NO_CARRIER                     = (RASBASE+79);
  ERROR_NO_DIALTONE                    = (RASBASE+80);
  ERROR_IN_COMMAND                     = (RASBASE+81);
  ERROR_WRITING_SECTIONNAME            = (RASBASE+82);
  ERROR_WRITING_DEVICETYPE             = (RASBASE+83);
  ERROR_WRITING_DEVICENAME             = (RASBASE+84);
  ERROR_WRITING_MAXCONNECTBPS          = (RASBASE+85);
  ERROR_WRITING_MAXCARRIERBPS          = (RASBASE+86);
  ERROR_WRITING_USAGE                  = (RASBASE+87);
  ERROR_WRITING_DEFAULTOFF             = (RASBASE+88);
  ERROR_READING_DEFAULTOFF             = (RASBASE+89);
  ERROR_EMPTY_INI_FILE                 = (RASBASE+90);
  ERROR_AUTHENTICATION_FAILURE         = (RASBASE+91);
  ERROR_PORT_OR_DEVICE                 = (RASBASE+92);
  ERROR_NOT_BINARY_MACRO               = (RASBASE+93);
  ERROR_DCB_NOT_FOUND                  = (RASBASE+94);
  ERROR_STATE_MACHINES_NOT_STARTED     = (RASBASE+95);
  ERROR_STATE_MACHINES_ALREADY_STARTED = (RASBASE+96);
  ERROR_PARTIAL_RESPONSE_LOOPING       = (RASBASE+97);
  ERROR_UNKNOWN_RESPONSE_KEY           = (RASBASE+98);
  ERROR_RECV_BUF_FULL                  = (RASBASE+99);
  ERROR_CMD_TOO_LONG                   = (RASBASE+100);
  ERROR_UNSUPPORTED_BPS                = (RASBASE+101);
  ERROR_UNEXPECTED_RESPONSE            = (RASBASE+102);
  ERROR_INTERACTIVE_MODE               = (RASBASE+103);
  ERROR_BAD_CALLBACK_NUMBER            = (RASBASE+104);
  ERROR_INVALID_AUTH_STATE             = (RASBASE+105);
  ERROR_WRITING_INITBPS                = (RASBASE+106);
  ERROR_X25_DIAGNOSTIC                 = (RASBASE+107);
  ERROR_ACCT_EXPIRED                   = (RASBASE+108);
  ERROR_CHANGING_PASSWORD              = (RASBASE+109);
  ERROR_OVERRUN                        = (RASBASE+110);
  ERROR_RASMAN_CANNOT_INITIALIZE	     = (RASBASE+111);
  ERROR_BIPLEX_PORT_NOT_AVAILABLE      = (RASBASE+112);
  ERROR_NO_ACTIVE_ISDN_LINES           = (RASBASE+113);
  ERROR_NO_ISDN_CHANNELS_AVAILABLE     = (RASBASE+114);
  ERROR_TOO_MANY_LINE_ERRORS           = (RASBASE+115);
  ERROR_IP_CONFIGURATION               = (RASBASE+116);
  ERROR_NO_IP_ADDRESSES                = (RASBASE+117);
  ERROR_PPP_TIMEOUT                    = (RASBASE+118);
  ERROR_PPP_REMOTE_TERMINATED          = (RASBASE+119);
  ERROR_PPP_NO_PROTOCOLS_CONFIGURED    = (RASBASE+120);
  ERROR_PPP_NO_RESPONSE                = (RASBASE+121);
  ERROR_PPP_INVALID_PACKET             = (RASBASE+122);
  ERROR_PHONE_NUMBER_TOO_LONG          = (RASBASE+123);
  ERROR_IPXCP_NO_DIALOUT_CONFIGURED    = (RASBASE+124);
  ERROR_IPXCP_NO_DIALIN_CONFIGURED     = (RASBASE+125);
  ERROR_IPXCP_DIALOUT_ALREADY_ACTIVE   = (RASBASE+126);
  ERROR_ACCESSING_TCPCFGDLL            = (RASBASE+127);
  ERROR_NO_IP_RAS_ADAPTER              = (RASBASE+128);
  ERROR_SLIP_REQUIRES_IP               = (RASBASE+129);
  ERROR_PROJECTION_NOT_COMPLETE        = (RASBASE+130);
  ERROR_PROTOCOL_NOT_CONFIGURED        = (RASBASE+131);
  ERROR_PPP_NOT_CONVERGING             = (RASBASE+132);
  ERROR_PPP_CP_REJECTED                = (RASBASE+133);
  ERROR_PPP_LCP_TERMINATED             = (RASBASE+134);
  ERROR_PPP_REQUIRED_ADDRESS_REJECTED  = (RASBASE+135);
  ERROR_PPP_NCP_TERMINATED             = (RASBASE+136);
  ERROR_PPP_LOOPBACK_DETECTED          = (RASBASE+137);
  ERROR_PPP_NO_ADDRESS_ASSIGNED        = (RASBASE+138);
  ERROR_CANNOT_USE_LOGON_CREDENTIALS   = (RASBASE+139);
  ERROR_TAPI_CONFIGURATION             = (RASBASE+140);
  ERROR_NO_LOCAL_ENCRYPTION            = (RASBASE+141);
  ERROR_NO_REMOTE_ENCRYPTION           = (RASBASE+142);
  ERROR_REMOTE_REQUIRES_ENCRYPTION     = (RASBASE+143);
  ERROR_IPXCP_NET_NUMBER_CONFLICT      = (RASBASE+144);
  ERROR_INVALID_SMM                    = (RASBASE+145);
  ERROR_SMM_UNINITIALIZED              = (RASBASE+146);
  ERROR_NO_MAC_FOR_PORT                = (RASBASE+147);
  ERROR_SMM_TIMEOUT                    = (RASBASE+148);
  ERROR_BAD_PHONE_NUMBER               = (RASBASE+149);
  ERROR_WRONG_MODULE                   = (RASBASE+150);

  RASBASEEND                           = (RASBASE+150);

(* Copyright (c) 1995, Microsoft Corporation, all rights reserved
**
** rnaph.h  (to be merged with ras.h)
**
** Remote Access external API
** Public header for external API clients
**)

const

  RAS_MaxAreaCode   =  10;
  RAS_MaxPadType    =  32;
  RAS_MaxX25Address = 200;
  RAS_MaxFacilities = 200;
  RAS_MaxUserData   = 200;


type

(* Describes a RAS IP Address
*)
  LPRasIPAddr = ^TRasIPAddr;
  TRasIPAddr = record
    a, b, c, d: Byte;
  end;

(* Describes a RAS phonebook entry
*)
  LPRasEntryA = ^TRasEntryA;
  TRasEntryA = record
    dwSize,
    dwfOptions,
    dwCountryID,
    dwCountryCode: Longint;
    szAreaCode: array[0.. RAS_MaxAreaCode] of AnsiChar;
    szLocalPhoneNumber: array[0..RAS_MaxPhoneNumber] of AnsiChar;
    dwAlternatesOffset: Longint;
    ipaddr,
    ipaddrDns,
    ipaddrDnsAlt,
    ipaddrWins,
    ipaddrWinsAlt: TRasIPAddr;
    dwFrameSize,
    dwfNetProtocols,
    dwFramingProtocol: Longint;
    szScript: Array[0..MAX_PATH - 1] of AnsiChar;
    szAutodialDll: Array [0..MAX_PATH - 1] of AnsiChar;
    szAutodialFunc: Array [0..MAX_PATH - 1] of AnsiChar;
    szDeviceType: Array [0..RAS_MaxDeviceType] of AnsiChar;
    szDeviceName: Array [0..RAS_MaxDeviceName] of AnsiChar;
    szX25PadType: Array [0..RAS_MaxPadType] of AnsiChar;
    szX25Address: Array [0..RAS_MaxX25Address] of AnsiChar;
    szX25Facilities: Array [0..RAS_MaxFacilities] of AnsiChar;
    szX25UserData: Array [0..RAS_MaxUserData] of AnsiChar;
    dwChannels,
    dwReserved1,
    dwReserved2: Longint;
  end;

  LPRasEntryW = ^TRasEntryW;
  TRasEntryW = record
    dwSize,
    dwfOptions,
    dwCountryID,
    dwCountryCode: Longint;
    szAreaCode: array[0.. RAS_MaxAreaCode] of WideChar;
    szLocalPhoneNumber: array[0..RAS_MaxPhoneNumber] of WideChar;
    dwAlternatesOffset: Longint;
    ipaddr,
    ipaddrDns,
    ipaddrDnsAlt,
    ipaddrWins,
    ipaddrWinsAlt: TRasIPAddr;
    dwFrameSize,
    dwfNetProtocols,
    dwFramingProtocol: Longint;
    szScript: Array[0..MAX_PATH - 1] of WideChar;
    szAutodialDll: Array [0..MAX_PATH - 1] of WideChar;
    szAutodialFunc: Array [0..MAX_PATH - 1] of WideChar;
    szDeviceType: Array [0..RAS_MaxDeviceType] of WideChar;
    szDeviceName: Array [0..RAS_MaxDeviceName] of WideChar;
    szX25PadType: Array [0..RAS_MaxPadType] of WideChar;
    szX25Address: Array [0..RAS_MaxX25Address] of WideChar;
    szX25Facilities: Array [0..RAS_MaxFacilities] of WideChar;
    szX25UserData: Array [0..RAS_MaxUserData] of WideChar;
    dwChannels,
    dwReserved1,
    dwReserved2: Longint;
  end;

  LPRasEntry = ^TRasEntry;
  TRasEntry = TRasEntryA;

(* Describes Country Information
*)
  LPRasCtryInfo = ^TRasCtryInfo;
  TRasCtryInfo = record
    dwSize,
    dwCountryID,
    dwNextCountryID,
    dwCountryCode,
    dwCountryNameOffset: Longint;
  end;


(* Describes RAS Device Information
*)
  LPRasDevInfoA = ^TRasDevInfoA;
  TRasDevInfoA = record
    dwSize: Longint;
    szDeviceType: Array[0..RAS_MaxDeviceType] of AnsiChar;
    szDeviceName: Array[0..RAS_MaxDeviceName] of AnsiChar;
  end;

  LPRasDevInfoW = ^TRasDevInfoW;
  TRasDevInfoW = record
    dwSize: Longint;
    szDeviceType: Array[0..RAS_MaxDeviceType] of WideChar;
    szDeviceName: Array[0..RAS_MaxDeviceName] of WideChar;
  end;

  LPRasDevInfo = ^TRasDevInfo;
  TRasDevInfo = TRasDevInfoA;

const

(* TRasEntry 'dwfOptions' bit flags.
*)
  RASEO_UseCountryAndAreaCodes = $00000001;
  RASEO_SpecificIpAddr         = $00000002;
  RASEO_SpecificNameServers    = $00000004;
  RASEO_IpHeaderCompression    = $00000008;
  RASEO_RemoteDefaultGateway   = $00000010;
  RASEO_DisableLcpExtensions   = $00000020;
  RASEO_TerminalBeforeDial     = $00000040;
  RASEO_TerminalAfterDial      = $00000080;
  RASEO_ModemLights            = $00000100;
  RASEO_SwCompression          = $00000200;
  RASEO_RequireEncryptedPw     = $00000400;
  RASEO_RequireMsEncryptedPw   = $00000800;
  RASEO_RequireDataEncryption  = $00001000;
  RASEO_NetworkLogon           = $00002000;
  RASEO_UseLogonCredentials    = $00004000;
  RASEO_PromoteAlternates      = $00008000;
  RASEO_SecureLocalFiles       = $00010000; //NT Only



(* TRasEntry 'dwfNetProtocols' bit flags. (session negotiated protocols)
*)
  RASNP_Netbeui = $00000001;  // Negotiate NetBEUI
  RASNP_Ipx     = $00000002;  // Negotiate IPX
  RASNP_Ip      = $00000004;  // Negotiate TCP/IP


(* TRasEntry 'dwFramingProtocols' (framing protocols used by the server)
*)
  RASFP_Ppp  = $00000001;  // Point-to-Point Protocol (PPP)
  RASFP_Slip = $00000002;  // Serial Line Internet Protocol (SLIP)
  RASFP_Ras  = $00000004;  // Microsoft proprietary protocol


(* TRasEntry 'szDeviceType' strings
*)
  RASDT_Modem = 'modem';     // Modem
  RASDT_Isdn  = 'isdn';      // ISDN
  RASDT_X25   = 'x25';      // X.25


(* RAS functions found in RNAPH.DLL
*)
Type
  TRasValidateEntryNameA = function(
    lpszPhonebook,
    szEntry: PAnsiChar
    ): Longint; stdcall;
  TRasValidateEntryNameW = function (
    lpszPhonebook,
    szEntry: PWideChar
    ): Longint; stdcall;
  TRasValidateEntryName = function (
    lpszPhonebook,
    szEntry: PAnsiChar
    ): Longint; stdcall;

  TRasRenameEntryA = function (
    lpszPhonebook,
    szEntryOld,
    szEntryNew: PAnsiChar
    ): Longint; stdcall;
  TRasRenameEntryW = function (
    lpszPhonebook,
    szEntryOld,
    szEntryNew: PWideChar
    ): Longint; stdcall;
  TRasRenameEntry = function (
    lpszPhonebook,
    szEntryOld,
    szEntryNew: PAnsiChar
    ): Longint; stdcall;

  TRasDeleteEntryA = function (
    lpszPhonebook,
    szEntry: PAnsiChar
    ): Longint; stdcall;
  TRasDeleteEntryW = function (
    lpszPhonebook,
    szEntry: PWideChar
    ): Longint; stdcall;
  TRasDeleteEntry = function (
    lpszPhonebook,
    szEntry: PAnsiChar
    ): Longint; stdcall;

  TRasGetEntryPropertiesA = function (
    lpszPhonebook,
    szEntry: PAnsiChar;
    lpbEntry: Pointer;
    var lpdwEntrySize: Longint;
    lpbDeviceInfo: Pointer;
    var lpdwDeviceInfoSize: Longint
    ): Longint; stdcall;
  TRasGetEntryPropertiesW = function (
    lpszPhonebook,
    szEntry: PWideChar;
    lpbEntry: Pointer;
    var lpdwEntrySize: Longint;
    lpbDeviceInfo: Pointer;
    var lpdwDeviceInfoSize: Longint
    ): Longint; stdcall;
  function RasGetEntryProperties(
    lpszPhonebook,
    szEntry: PAnsiChar;
    lpbEntry: Pointer;
    var lpdwEntrySize: Longint;
    lpbDeviceInfo: Pointer;
    var lpdwDeviceInfoSize: Longint
    ): Longint; stdcall;

type
  TRasSetEntryPropertiesA = function (
    lpszPhonebook,
    szEntry: PAnsiChar;
    lpbEntry: Pointer;
    dwEntrySize: Longint;
    lpbDeviceInfo: Pointer;
    dwDeviceInfoSize: Longint
    ): Longint; stdcall;
  TRasSetEntryPropertiesW = function (
    lpszPhonebook,
    szEntry: PWideChar;
    lpbEntry: Pointer;
    dwEntrySize: Longint;
    lpbDeviceInfo: Pointer;
    dwDeviceInfoSize: Longint
    ): Longint; stdcall;

  TRasGetCountryInfoA = function (
    var lpCtryInfo: TRasCtryInfo;
    var lpdwSize: Longint
    ): Longint; stdcall;
  TRasGetCountryInfoW = function (
    var lpCtryInfo: TRasCtryInfo;
    var lpdwSize: Longint
    ): Longint; stdcall;
  TRasEnumDevicesA = function (
    lpBuff: LpRasDevInfoA;
    var lpcbSize: Longint;
    var lpcDevices: Longint
    ): Longint; stdcall;
  TRasEnumDevicesW = function (
    lpBuff: LpRasDevInfoW;
    var lpcbSize: Longint;
    var lpcDevices: Longint
    ): Longint; stdcall;

  function RasGetCountryInfo(
    var lpCtryInfo: TRasCtryInfo;
    var lpdwSize: Longint
    ): Longint; stdcall;

  function RasEnumDevices(
    lpBuff: LpRasDevInfo;
    var lpcbSize: Longint;
    var lpcDevices: Longint
    ): Longint; stdcall;
  function RasSetEntryProperties(
    lpszPhonebook,
    szEntry: PAnsiChar;
    lpbEntry: Pointer;
    dwEntrySize: Longint;
    lpbDeviceInfo: Pointer;
    dwDeviceInfoSize: Longint
    ): Longint; stdcall;
  function InitRas: Boolean;
  function CloseRas: Boolean;



var
  RasCreatePhonebookEntryA: TRasCreatePhonebookEntryA;
  RasCreatePhonebookEntryW: TRasCreatePhonebookEntryW;
  RasCreatePhonebookEntry: TRasCreatePhonebookEntry;
  RasDialA: TRasDialA;
  RasDialW: TRasDialW;
  RasDial: TRasDial;
  RasEditPhonebookEntryA: TRasEditPhonebookEntryA;
  RasEditPhonebookEntryW: TRasEditPhonebookEntryW;
  RasEditPhonebookEntry: TRasEditPhonebookEntry;
  RasEnumConnectionsA: TRasEnumConnectionsA;
  RasEnumConnectionsW: TRasEnumConnectionsW;
  RasEnumConnections: TRasEnumConnections;
  RasEnumEntriesA: TRasEnumEntriesA;
  RasEnumEntriesW: TRasEnumEntriesW;
  RasEnumEntries: TRasEnumEntries;
  RasGetConnectStatusA: TRasGetConnectStatusA;
  RasGetConnectStatusW: TRasGetConnectStatusW;
  RasGetConnectStatus: TRasGetConnectStatus;
  RasGetEntryDialParamsA: TRasGetEntryDialParamsA;
  RasGetEntryDialParamsW: TRasGetEntryDialParamsW;
  RasGetEntryDialParams: TRasGetEntryDialParams;
  RasGetErrorStringA: TRasGetErrorStringA;
  RasGetErrorStringW: TRasGetErrorStringW;
  RasGetErrorString: TRasGetErrorString;
  RasGetProjectionInfoA: TRasGetProjectionInfoA;
  RasGetProjectionInfoW: TRasGetProjectionInfoW;
  RasGetProjectionInfo: TRasGetProjectionInfo;
  RasHangUpA: TRasHangUpA;
  RasHangUpW: TRasHangUpW;
  RasHangUp: TRasHangUp;
  RasSetEntryDialParamsA: TRasSetEntryDialParamsA;
  RasSetEntryDialParamsW: TRasSetEntryDialParamsW;
  RasSetEntryDialParams: TRasSetEntryDialParams;

  RasValidateEntryNameA: TRasValidateEntryNameA;
  RasValidateEntryNameW: TRasValidateEntryNameW;
  // function RasValidateEntryName;   external 'rasapi32.dll' name 'RasValidateEntryNameA';
  RasRenameEntryA: TRasRenameEntryA;
  RasRenameEntryW: TRasRenameEntryW;
  // function RasRenameEntry;         external 'rasapi32.dll' name 'RasRenameEntryA';
  RasDeleteEntry: TRasDeleteEntry;
  RasDeleteEntryA: TRasDeleteEntryA;
  RasDeleteEntryW: TRasDeleteEntryW;
  // function RasDeleteEntry;         external 'rasapi32.dll' name 'RasDeleteEntryA';
  RasGetEntryPropertiesA: TRasGetEntryPropertiesA;
  RasGetEntryPropertiesW: TRasGetEntryPropertiesW;
  // function RasGetEntryProperties;  external 'rasapi32.dll' name 'RasGetEntryPropertiesA';
  RasSetEntryPropertiesA: TRasSetEntryPropertiesA;
  RasSetEntryPropertiesW: TRasSetEntryPropertiesW;
  // function RasSetEntryProperties;  external 'rasapi32.dll' name 'RasSetEntryPropertiesA';
  RasGetCountryInfoA: TRasGetCountryInfoA;
  RasGetCountryInfoW: TRasGetCountryInfoW;
  // function RasGetCountryInfo;      external 'rasapi32.dll' name 'RasGetCountryInfoA';
  RasEnumDevicesA: TRasEnumDevicesA;
  RasEnumDevicesW: TRasEnumDevicesW;
  //RasEnumDevices: TRasEnumDevices;


var
  rnaph_initialized: Boolean = False;
  is_rnaph: Boolean = False;
  lib: HModule;
  hLib: THandle;

implementation

function rnaph_(const func: String): Pointer;
begin
  if not rnaph_initialized then
  begin
    // Try first with RASAPI32.DLL
    lib := LoadLibrary('rasapi32.dll');
    if lib <> 0 then
    begin
      Result := GetProcAddress(lib, PChar(func + 'A'));
      if Result <> nil then
      begin
        rnaph_initialized := True;
        exit;
      end;
    end
    else
      raise Exception.Create('Error opening rasapi32.dll');
    // function not found - try rnaph.dll
    lib := LoadLibrary('rnaph.dll');
    if lib <> 0 then
      begin
      Result := GetProcAddress(lib, PChar(func));
      if Result <> nil then
        begin
        rnaph_initialized := True;
        is_rnaph := True;
        Exit;
        end
      else
        raise Exception.Create('Function ' + func + ' not found!');
      end
    else
      raise Exception.Create('Error opening rnaph.dll');
    end
  else
    begin
    if is_rnaph then
      Result := GetProcAddress(lib, PChar(func))
    else
      Result := GetProcAddress(lib, PChar(func + 'A'));
    if Result = nil then
      raise Exception.Create('Function ' + func + ' not found!');
    end;
  end;

function RasValidateEntryName(lpszPhonebook, szEntry: PAnsiChar): Longint;
  var
    f: Function(lpszPhonebook, szEntry: PAnsiChar): Longint; stdcall;
  begin
  @f := rnaph_('RasValidateEntryName');
  Result := f(lpszPhonebook, szEntry);
  end;

function RasRenameEntry(lpszPhonebook, szEntryOld, szEntryNew: PAnsiChar): Longint;
  var
    f: function(lpszPhonebook, szEntryOld, szEntryNew: PAnsiChar): Longint; stdcall;
  begin
  @f := rnaph_('RasRenameEntry');
  Result := f(lpszPhonebook, szEntryOld, szEntryNew);
  end;
(*
function RasDeleteEntry(lpszPhonebook, szEntry: PAnsiChar): Longint;
  var
    f: function(lpszPhonebook, szEntry: PAnsiChar): Longint; stdcall;
  begin
  @f := rnaph_('RasDeleteEntry');
  Result := f(lpszPhonebook, szEntry);
  end;
*)
function RasGetEntryProperties(lpszPhonebook, szEntry: PAnsiChar; lpbEntry: Pointer;
    var lpdwEntrySize: Longint; lpbDeviceInfo: Pointer;
    var lpdwDeviceInfoSize: Longint): Longint;
  var
    f: function(lpszPhonebook, szEntry: PAnsiChar; lpbEntry: Pointer;
        var lpdwEntrySize: Longint; lpbDeviceInfo: Pointer;
        var lpdwDeviceInfoSize: Longint): Longint; stdcall;
  begin
  @f := rnaph_('RasGetEntryProperties');
  Result := f(lpszPhonebook, szEntry, lpbEntry, lpdwEntrySize, lpbDeviceInfo, lpdwDeviceInfoSize);
  end;

function RasSetEntryProperties(lpszPhonebook, szEntry: PAnsiChar;
	  lpbEntry: Pointer; dwEntrySize: Longint; lpbDeviceInfo: Pointer;
    dwDeviceInfoSize: Longint): Longint;
  var
    f: function(lpszPhonebook, szEntry: PAnsiChar;
    	  lpbEntry: Pointer; dwEntrySize: Longint; lpbDeviceInfo: Pointer;
        dwDeviceInfoSize: Longint): Longint; stdcall;
  begin
  @f := rnaph_('RasSetEntryProperties');
  Result := f(lpszPhonebook, szEntry, lpbEntry, dwEntrySize, lpbDeviceInfo, dwDeviceInfoSize);
  end;

function RasGetCountryInfo(var lpCtryInfo: TRasCtryInfo;
    var lpdwSize: Longint): Longint;
  var
    f: function(var lpCtryInfo: TRasCtryInfo;
        var lpdwSize: Longint): Longint; stdcall;
  begin
  @f := rnaph_('RasGetCountryInfo');
  Result := f(lpCtryInfo, lpdwSize);
  end;

function RasEnumDevices(lpBuff: LpRasDevInfo; var lpcbSize: Longint;
    var lpcDevices: Longint): Longint;
  var
    f: function(lpBuff: LpRasDevInfo; var lpcbSize: Longint;
        var lpcDevices: Longint): Longint; stdcall;
  begin
  @f := rnaph_('RasEnumDevices');
  Result := f(lpBuff, lpcbSize, lpcDevices);
  end;

function InitRas: Boolean;
begin
  hLib := LoadLibrary('rasapi32.dll');
  if (hLib = 0) then
  begin
    Result := False;
    Exit;
  end;
  Result := True;
  try
    RasCreatePhonebookEntryA := TRasCreatePhonebookEntryA(GetProcAddress(hLib, 'RasCreatePhonebookEntryA'));
  except end;
  try
    RasCreatePhonebookEntryW := TRasCreatePhonebookEntryW(GetProcAddress(hLib, 'RasCreatePhonebookEntryW'));
  except end;
  try
    RasCreatePhonebookEntry := TRasCreatePhonebookEntry(GetProcAddress(hLib, 'RasCreatePhonebookEntryA'));
  except end;
  try
    RasDialA := TRasDialA(GetProcAddress(hLib, 'RasDialA'));
  except end;
  try
    RasDialW := TRasDialW(GetProcAddress(hLib, 'RasDialW'));
  except end;
  try
    RasDial := TRasDial(GetProcAddress(hLib, 'RasDialA'));
  except end;
  try
    RasEditPhonebookEntryA := TRasEditPhonebookEntryA(GetProcAddress(hLib, 'RasEditPhonebookEntryA'));
  except end;
  try
    RasEditPhonebookEntryW := TRasEditPhonebookEntryW(GetProcAddress(hLib, 'RasEditPhonebookEntryW'));
  except end;
  try
    RasEditPhonebookEntry := TRasEditPhonebookEntry(GetProcAddress(hLib, 'RasEditPhonebookEntryA'));
  except end;
  try
    RasEnumConnectionsA := TRasEnumConnectionsA(GetProcAddress(hLib, 'RasEnumConnectionsA'));
  except end;
  try
    RasEnumConnectionsW := TRasEnumConnectionsW(GetProcAddress(hLib, 'RasEnumConnectionsW'));
  except end;
  try
    RasEnumConnections := TRasEnumConnections(GetProcAddress(hLib, 'RasEnumConnectionsA'));
  except end;
  try
    RasEnumEntriesA := TRasEnumEntriesA(GetProcAddress(hLib, 'RasEnumEntriesA'));
  except end;
  try
    RasEnumEntriesW := TRasEnumEntriesW(GetProcAddress(hLib, 'RasEnumEntriesW'));
  except end;
  try
    RasEnumEntries := TRasEnumEntries(GetProcAddress(hLib, 'RasEnumEntriesA'));
  except end;
  try
    RasGetConnectStatusA := TRasGetConnectStatusA(GetProcAddress(hLib, 'RasGetConnectStatusA'));
  except end;
  try
    RasGetConnectStatusW := TRasGetConnectStatusW(GetProcAddress(hLib, 'RasGetConnectStatusW'));
  except end;
  try
    RasGetConnectStatus := TRasGetConnectStatus(GetProcAddress(hLib, 'RasGetConnectStatusA'));
  except end;
  try
    RasGetEntryDialParamsA := TRasGetEntryDialParamsA(GetProcAddress(hLib, 'RasGetEntryDialParamsA'));
  except end;
  try
    RasGetEntryDialParamsW := TRasGetEntryDialParamsW(GetProcAddress(hLib, 'RasGetEntryDialParamsW'));
  except end;
  try
    RasGetEntryDialParams := TRasGetEntryDialParams(GetProcAddress(hLib, 'RasGetEntryDialParamsA'));
  except end;
  try
    RasGetErrorStringA := TRasGetErrorStringA(GetProcAddress(hLib, 'RasGetErrorStringA'));
  except end;
  try
    RasGetErrorStringW := TRasGetErrorStringW(GetProcAddress(hLib, 'RasGetErrorStringW'));
  except end;
  try
    RasGetErrorString := TRasGetErrorString(GetProcAddress(hLib, 'RasGetErrorStringA'));
  except end;
  try
    RasGetProjectionInfoA := TRasGetProjectionInfoA(GetProcAddress(hLib, 'RasGetProjectionInfoA'));
  except end;
  try
    RasGetProjectionInfoW := TRasGetProjectionInfoW(GetProcAddress(hLib, 'RasGetProjectionInfoW'));
  except end;
  try
    RasGetProjectionInfo := TRasGetProjectionInfo(GetProcAddress(hLib, 'RasGetProjectionInfoA'));
  except end;
  try
    RasHangUpA := TRasHangUpA(GetProcAddress(hLib, 'RasHangUpA'));
  except end;
  try
    RasHangUpW := TRasHangUpW(GetProcAddress(hLib, 'RasHangUpW'));
  except end;
  try
    RasHangUp := TRasHangUp(GetProcAddress(hLib, 'RasHangUpA'));
  except end;
  try
    RasSetEntryDialParamsA := TRasSetEntryDialParamsA(GetProcAddress(hLib, 'RasSetEntryDialParamsA'));
  except end;
  try
    RasSetEntryDialParamsW := TRasSetEntryDialParamsW(GetProcAddress(hLib, 'RasSetEntryDialParamsW'));
  except end;
  try
    RasSetEntryDialParams := TRasSetEntryDialParams(GetProcAddress(hLib, 'RasSetEntryDialParamsA'));
  except end;

  {* These are subject to change, since RNAPH.DLL will be removed.
  ** The 'A' and 'W' functions are present in Windows NT 4.0
  *}
  RasValidateEntryNameA := TRasValidateEntryNameA(GetProcAddress(hLib, 'RasValidateEntryNameA'));
  RasValidateEntryNameW := TRasValidateEntryNameW(GetProcAddress(hLib, 'RasValidateEntryNameW'));
  // function RasValidateEntryName;   external 'rasapi32.dll' name 'RasValidateEntryNameA';
  RasRenameEntryA := TRasRenameEntryA(GetProcAddress(hLib, 'RasRenameEntryA'));
  RasRenameEntryW := TRasRenameEntryW(GetProcAddress(hLib, 'RasRenameEntryW'));
  // function RasRenameEntry;         external 'rasapi32.dll' name 'RasRenameEntryA';
  RasDeleteEntry := TRasDeleteEntryA(GetProcAddress(hLib, 'RasDeleteEntryA'));
  RasDeleteEntryA := TRasDeleteEntryA(GetProcAddress(hLib, 'RasDeleteEntryA'));
  RasDeleteEntryW := TRasDeleteEntryW(GetProcAddress(hLib, 'RasDeleteEntryW'));
  // function RasDeleteEntry;         external 'rasapi32.dll' name 'RasDeleteEntryA';
  RasGetEntryPropertiesA := TRasGetEntryPropertiesA(GetProcAddress(hLib, 'RasGetEntryPropertiesA'));
  RasGetEntryPropertiesW := TRasGetEntryPropertiesW(GetProcAddress(hLib, 'RasGetEntryPropertiesW'));
  // function RasGetEntryProperties;  external 'rasapi32.dll' name 'RasGetEntryPropertiesA';
  RasSetEntryPropertiesA := TRasSetEntryPropertiesA(GetProcAddress(hLib, 'RasSetEntryPropertiesA'));
  RasSetEntryPropertiesW := TRasSetEntryPropertiesW(GetProcAddress(hLib, 'RasSetEntryPropertiesW'));
  // function RasSetEntryProperties;  external 'rasapi32.dll' name 'RasSetEntryPropertiesA';
  RasGetCountryInfoA := TRasGetCountryInfoA(GetProcAddress(hLib, 'RasGetCountryInfoA'));
  RasGetCountryInfoW := TRasGetCountryInfoW(GetProcAddress(hLib, 'RasGetCountryInfoW'));
  // function RasGetCountryInfo;      external 'rasapi32.dll' name 'RasGetCountryInfoA';
  RasEnumDevicesA := TRasEnumDevicesA(GetProcAddress(hLib, 'RasEnumDevicesA'));
  RasEnumDevicesW := TRasEnumDevicesW(GetProcAddress(hLib, 'RasEnumDevicesW'));
  // function RasEnumDevices;         external 'rasapi32.dll' name 'RasEnumDevicesA';
end;

function CloseRas: Boolean;
begin
  if (hlib <> 0) then
    FreeLibrary(hLib);
  Result := true;
end;

initialization
finalization
  if rnaph_initialized and is_rnaph then
    FreeLibrary(lib);
end.
